@@ -72,6 +72,7 @@
 #include "mupdate_err.h"
 #include "mutex.h"
 #include "prot.h" /* for PROT_BUFSIZE */
+#include "strarray.h"
 #include "userdeny.h"
 #include "util.h"
 #include "xmalloc.h"
@@ -103,11 +104,12 @@ struct cyrusdb_backend *config_ptscache_db;
 struct cyrusdb_backend *config_statuscache_db;
 struct cyrusdb_backend *config_userdeny_db;
 
-#define MAX_SESSIONID_SIZE 256
 char session_id_buf[MAX_SESSIONID_SIZE];
 int session_id_time = 0;
 int session_id_count = 0;
 
+strarray_t *suppressed_capabilities = NULL;
+
 /* Called before a cyrus application starts (but after command line parameters
  * are read) */
 int cyrus_init(const char *alt_config, const char *ident, unsigned flags)
@@ -182,6 +184,10 @@ int cyrus_init(const char *alt_config, const char *ident, unsigned flags)
 
     config_metapartition_files = config_getbitfield(IMAPOPT_METAPARTITION_FILES);
 
+    val = config_getstring(IMAPOPT_SUPPRESS_CAPABILITIES);
+    if (val)
+	suppressed_capabilities = strarray_split(val, NULL, 0);
+
     if (!cyrus_init_nodb) {
 	/* lookup the database backends */
 	config_mboxlist_db =
@@ -223,6 +229,8 @@ int cyrus_init(const char *alt_config, const char *ident, unsigned flags)
 				  config_getswitch(IMAPOPT_FULLDIRHASH));
 	libcyrus_config_setstring(CYRUSOPT_PTSCACHE_DB,
 				  config_getstring(IMAPOPT_PTSCACHE_DB));
+	libcyrus_config_setstring(CYRUSOPT_PTSCACHE_DB_PATH,
+				  config_getstring(IMAPOPT_PTSCACHE_DB_PATH));
 	libcyrus_config_setstring(CYRUSOPT_PTLOADER_SOCK,
 				  config_getstring(IMAPOPT_PTLOADER_SOCK));
 	libcyrus_config_setswitch(CYRUSOPT_VIRTDOMAINS,
@@ -459,7 +467,7 @@ int mysasl_canon_user(sasl_conn_t *conn,
 	sasl_seterror(conn, 0, "buffer overflow while canonicalizing");
 	return SASL_BUFOVER;
     }
-    memcpy(out, user, ulen);
+    memmove(out, user, ulen);
     out[ulen] = '\0';
 
     canonuser = canonify_userid(out, NULL, (int*) context);
@@ -689,6 +697,7 @@ int shutdown_file(char *buf, int size)
     FILE *f;
     static char shutdownfilename[1024] = "";
     char *p;
+    char tmpbuf[1024];
     
     if (!shutdownfilename[0])
 	snprintf(shutdownfilename, sizeof(shutdownfilename), 
@@ -697,6 +706,11 @@ int shutdown_file(char *buf, int size)
     f = fopen(shutdownfilename, "r");
     if (!f) return 0;
 
+    if (!buf) {
+	buf = tmpbuf;
+	size = sizeof(tmpbuf);
+    }
+
     if (!fgets(buf, size, f)) {
 	*buf = '\0';
 
@@ -779,7 +793,7 @@ char *find_free_partition(unsigned long *tavail)
 void session_new_id()
 {
     const char *base;
-    int now = time(NULL);    
+    int now = time(NULL);
     if (now != session_id_time) {
         session_id_time = now;
         session_id_count = 0;
@@ -798,3 +812,33 @@ const char *session_id()
         session_new_id();
     return (const char *)session_id_buf;
 }
+
+/* parse sessionid out of protocol answers */
+void parse_sessionid(const char *str, char *sessionid)
+{
+    char *sp, *ep;
+    int len;
+
+    if ((str) && (sp = strstr(str, "SESSIONID=<")) && (ep = strchr(sp, '>')))
+    {
+	sp += 11;
+	len = ep - sp;
+	if (len < MAX_SESSIONID_SIZE)
+	{
+	    strncpy(sessionid, sp, len);
+	    ep = sessionid + len;
+	    *ep = '\0';
+	}
+	else
+	    strcpy(sessionid, "invalid");
+    }
+    else
+	strcpy(sessionid, "unknown");
+}
+
+int capa_is_disabled(const char *str)
+{
+    if (!suppressed_capabilities) return 0;
+
+    return (strarray_find_case(suppressed_capabilities, str, 0) >= 0);
+}
