@@ -39,7 +39,7 @@
  * AN ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING
  * OUT OF OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
  *
- * $Id: idled.c,v 1.27 2009/07/28 02:50:54 brong Exp $
+ * $Id: idled.c,v 1.28 2010/01/06 17:01:32 murch Exp $
  */
 
 #ifdef HAVE_CONFIG_H
@@ -47,8 +47,6 @@
 #endif
 
 #include <sys/types.h>
-#include <sys/socket.h>
-#include <sys/un.h>
 #include <syslog.h>
 #include <sys/stat.h>
 #include <stdlib.h>
@@ -59,12 +57,10 @@
 #include <signal.h>
 #include <fcntl.h>
 
-#include "idled.h"
+#include "idlemsg.h"
 #include "global.h"
 #include "mboxlist.h"
 #include "xmalloc.h"
-#include "xstrlcpy.h"
-#include "xstrlcat.h"
 #include "hash.h"
 #include "exitcodes.h"
 
@@ -79,14 +75,11 @@ static int debugmode = 0;
 static time_t idle_timeout;
 
 struct ientry {
-    pid_t pid;
+    struct sockaddr_un remote;
     time_t itime;
     struct ientry *next;
 };
 static struct hash_table itable;
-static struct ientry *ifreelist;
-static int itable_inc = 100;
-void idle_done(char *mboxname, pid_t pid);
 
 void fatal(const char *msg, int err)
 {
@@ -95,7 +88,7 @@ void fatal(const char *msg, int err)
     syslog(LOG_NOTICE, "exiting");
 
     cyrus_done();
-    
+
     exit(err);
 }
 
@@ -111,44 +104,20 @@ static int mbox_count_cb(void *rockp,
     return 0;
 }
 
-/* return a new 'ientry', either from the freelist or by malloc'ing it */
-static struct ientry *get_ientry(void)
-{
-    struct ientry *t;
-
-    if (!ifreelist) {
-	/* create child_table_inc more and add them to the freelist */
-	struct ientry *n;
-	int i;
-
-	n = xmalloc(itable_inc * sizeof(struct ientry));
-	ifreelist = n;
-	for (i = 0; i < itable_inc - 1; i++) {
-	    n[i].next = n + (i + 1);
-	}
-	/* i == child_table_inc - 1, last item in block */
-	n[i].next = NULL;
-    }
-
-    t = ifreelist;
-    ifreelist = ifreelist->next;
-
-    return t;
-}
-
-/* remove pid from list of those idling on mboxname */
-void idle_done(char *mboxname, pid_t pid)
+/* remove an ientry from list of those idling on mboxname */
+static void remove_ientry(const char *mboxname,
+			  const struct sockaddr_un *remote)
 {
     struct ientry *t, *p = NULL;
 
     t = (struct ientry *) hash_lookup(mboxname, &itable);
-    while (t && t->pid != pid) {
+    while (t && memcmp(&t->remote, remote, sizeof(*remote))) {
 	p = t;
 	t = t->next;
     }
     if (t) {
 	if (!p) {
-	    /* first pid in the linked list */
+	    /* first ientry in the linked list */
 
 	    p = t->next; /* remove node */
 
@@ -157,115 +126,157 @@ void idle_done(char *mboxname, pid_t pid)
 	    hash_insert(mboxname, p, &itable);
 	}
 	else {
-	    /* not the first pid in the linked list */
+	    /* not the first ientry in the linked list */
 
 	    p->next = t->next; /* remove node */
 	}
-	t->next = ifreelist; /* add to freelist */
-	ifreelist = t;
+	free(t);
     }
 }
 
-void process_msg(idle_data_t *idledata)
+
+
+static void process_message(struct sockaddr_un *remote, idle_message_t *msg)
 {
     struct ientry *t, *n;
+    int r;
 
-    switch (idledata->msg) {
-    case IDLE_INIT:
+    switch (msg->which) {
+    case IDLE_MSG_INIT:
 	if (verbose || debugmode)
-	    syslog(LOG_DEBUG, "imapd[%ld]: IDLE_INIT '%s'\n",
-		   idledata->pid, idledata->mboxname);
+	    syslog(LOG_DEBUG, "imapd[%s]: IDLE_MSG_INIT '%s'\n",
+		   idle_id_from_addr(remote), msg->mboxname);
 
-	/* add pid to list of those idling on mboxname */
-	t = (struct ientry *) hash_lookup(idledata->mboxname, &itable);
-	n = get_ientry();
-	n->pid = idledata->pid;
+	/* add an ientry to list of those idling on mboxname */
+	t = (struct ientry *) hash_lookup(msg->mboxname, &itable);
+	n = (struct ientry *) xzmalloc(sizeof(struct ientry));
+	n->remote = *remote;
 	n->itime = time(NULL);
 	n->next = t;
-	hash_insert(idledata->mboxname, n, &itable);
+	hash_insert(msg->mboxname, n, &itable);
 	break;
-	
-    case IDLE_NOTIFY:
-	if (verbose || debugmode)
-	    syslog(LOG_DEBUG, "IDLE_NOTIFY '%s'\n", idledata->mboxname);
 
-	/* send a message to all pids idling on mboxname */
-	t = (struct ientry *) hash_lookup(idledata->mboxname, &itable);
-	while (t) {
+    case IDLE_MSG_NOTIFY:
+	if (verbose || debugmode)
+	    syslog(LOG_DEBUG, "IDLE_MSG_NOTIFY '%s'\n", msg->mboxname);
+
+	/* send a message to all clients idling on mboxname */
+	t = (struct ientry *) hash_lookup(msg->mboxname, &itable);
+	for ( ; t ; t = n) {
+	    n = t->next;
 	    if ((t->itime + idle_timeout) < time(NULL)) {
 		/* This process has been idling for longer than the timeout
 		 * period, so it probably died.  Remove it from the list.
 		 */
 		if (verbose || debugmode)
-		    syslog(LOG_DEBUG, "    TIMEOUT %d\n", t->pid);
+		    syslog(LOG_DEBUG, "    TIMEOUT %s\n", idle_id_from_addr(&t->remote));
 
-		n = t;
-		t = t->next;
-		idle_done(idledata->mboxname, n->pid);
+		remove_ientry(msg->mboxname, &t->remote);
 	    }
 	    else { /* signal process to update */
 		if (verbose || debugmode)
-		    syslog(LOG_DEBUG, "    SIGUSR1 %d\n", t->pid);
+		    syslog(LOG_DEBUG, "    fwd NOTIFY %s\n", idle_id_from_addr(&t->remote));
 
-		kill(t->pid, SIGUSR1);
-		t = t->next;
+		/* forward the received msg onto our clients */
+		r = idle_send(&t->remote, msg);
+		if (r) {
+		    /* ENOENT can happen as result of a race between delivering
+		     * messages and shutting down imapd.  It indicates that the
+		     * imapd's socket was unlinked, which means that imapd went
+		     * through it's graceful shutdown path, so don't syslog. */
+		    if (r != ENOENT)
+			syslog(LOG_ERR, "IDLE: error sending message "
+					"NOTIFY to imapd %s for mailbox %s: %s, "
+					"forgetting.",
+					idle_id_from_addr(&t->remote),
+					msg->mboxname, error_message(r));
+		    if (verbose || debugmode)
+			syslog(LOG_DEBUG, "    forgetting %s\n", idle_id_from_addr(&t->remote));
+		    remove_ientry(msg->mboxname, &t->remote);
+		}
 	    }
 	}
 	break;
-	
-    case IDLE_DONE:
+
+    case IDLE_MSG_DONE:
 	if (verbose || debugmode)
-	    syslog(LOG_DEBUG, "imapd[%ld]: IDLE_DONE '%s'\n",
-		   idledata->pid, idledata->mboxname);
+	    syslog(LOG_DEBUG, "imapd[%s]: IDLE_MSG_DONE '%s'\n",
+		   idle_id_from_addr(remote), msg->mboxname);
 
-	/* remove pid from list of those idling on mboxname */
-	idle_done(idledata->mboxname, idledata->pid);
+	/* remove client from list of those idling on mboxname */
+	remove_ientry(msg->mboxname, remote);
 	break;
 
-    case IDLE_NOOP:
+    case IDLE_MSG_NOOP:
 	break;
-	
+
     default:
-	syslog(LOG_ERR, "unrecognized message: %lx", idledata->msg);
+	syslog(LOG_ERR, "unrecognized message: %lx", msg->which);
 	break;
     }
 }
 
-void idle_alert(char *key __attribute__((unused)),
-		void *data,
-		void *rock __attribute__((unused)))
+
+static void send_alert(const char *key,
+		       void *data,
+		       void *rock __attribute__((unused)))
 {
     struct ientry *t = (struct ientry *) data;
+    struct ientry *n;
+    idle_message_t msg;
+    int r;
+
+    msg.which = IDLE_MSG_ALERT;
+    strncpy(msg.mboxname, ".", sizeof(msg.mboxname));
+
+
+    for ( ; t ; t = n) {
+	n = t->next;
 
-    while (t) {
 	/* signal process to check ALERTs */
 	if (verbose || debugmode)
-	    syslog(LOG_DEBUG, "    SIGUSR2 %d\n", t->pid);
-	kill(t->pid, SIGUSR2);
+	    syslog(LOG_DEBUG, "    ALERT %s\n", idle_id_from_addr(&t->remote));
 
-	t = t->next;
+	r = idle_send(&t->remote, &msg);
+	if (r) {
+	    /* ENOENT can happen as result of a race between shutting
+	     * down idled and shutting down imapd.  It indicates that the
+	     * imapd's socket was unlinked, which means that imapd went
+	     * through it's graceful shutdown path, so don't syslog. */
+	    if (r != ENOENT)
+		syslog(LOG_ERR, "IDLE: error sending message "
+				"ALERT to imapd %s for mailbox %s: %s, "
+				"forgetting.",
+				idle_id_from_addr(&t->remote),
+				msg.mboxname, error_message(r));
+	    if (verbose || debugmode)
+		syslog(LOG_DEBUG, "    forgetting %s\n", idle_id_from_addr(&t->remote));
+	    remove_ientry(key, &t->remote);
+	}
     }
 }
 
+static void shut_down(int ec) __attribute__((noreturn));
+static void shut_down(int ec)
+{
+    hash_enumerate(&itable, send_alert, NULL);
+    idle_done_sock();
+    cyrus_done();
+    exit(ec);
+}
+
 int main(int argc, char **argv)
 {
-    char shutdownfilename[1024];
     char *p = NULL;
     int opt;
     int nmbox = 0;
-    int s, len;
+    int s;
     struct sockaddr_un local;
-    idle_data_t idledata;
-    struct sockaddr_un from;
-    socklen_t fromlen;
-    mode_t oldumask;
     fd_set read_set, rset;
     int nfds;
     struct timeval timeout;
     pid_t pid;
-    int fd;
     char *alt_config = NULL;
-    const char *idle_sock;
 
     p = getenv("CYRUS_VERBOSE");
     if (p) verbose = atoi(p) + 1;
@@ -285,28 +296,8 @@ int main(int argc, char **argv)
 	}
     }
 
-    /* fork unless we were given the -d option */
-    if (debugmode == 0) {
-	
-	pid = fork();
-	
-	if (pid == -1) {
-	    perror("fork");
-	    exit(1);
-	}
-	
-	if (pid != 0) { /* parent */
-	    exit(0);
-	}
-    }
-    /* child */
-
     cyrus_init(alt_config, "idled", 0);
 
-    /* get name of shutdown file */
-    snprintf(shutdownfilename, sizeof(shutdownfilename), "%s/msg/shutdown",
-	     config_dir);
-
     /* Set inactivity timer (convert from minutes to seconds) */
     idle_timeout = config_getint(IMAPOPT_TIMEOUT);
     if (idle_timeout < 30) idle_timeout = 30;
@@ -315,44 +306,39 @@ int main(int argc, char **argv)
     /* count the number of mailboxes */
     mboxlist_init(0);
     mboxlist_open(NULL);
-    config_mboxlist_db->foreach(mbdb, "", 0, NULL, &mbox_count_cb,
-				&nmbox, NULL);
+    mboxlist_allmbox("", &mbox_count_cb, &nmbox);
     mboxlist_close();
     mboxlist_done();
 
+    signals_set_shutdown(shut_down);
+    signals_add_handlers(0);
+
     /* create idle table -- +1 to avoid a zero value */
     construct_hash_table(&itable, nmbox + 1, 1);
-    ifreelist = NULL;
 
-    /* create socket we are going to use for listening */
-    if ((s = socket(AF_UNIX, SOCK_DGRAM, 0)) == -1) {
-	perror("socket");
+    if (!idle_make_server_address(&local) ||
+	!idle_init_sock(&local)) {
 	cyrus_done();
 	exit(1);
     }
+    s = idle_get_sock();
 
-    /* bind it to a local file */
-    local.sun_family = AF_UNIX;
-    idle_sock = config_getstring(IMAPOPT_IDLESOCKET);
-    if (idle_sock) {	
-	strlcpy(local.sun_path, idle_sock, sizeof(local.sun_path));
-    }
-    else {
-	strlcpy(local.sun_path, config_dir, sizeof(local.sun_path));
-	strlcat(local.sun_path, FNAME_IDLE_SOCK, sizeof(local.sun_path));
-    }
-    unlink(local.sun_path);
-    len = sizeof(local.sun_family) + strlen(local.sun_path) + 1;
+    /* fork unless we were given the -d option or we're running as a daemon */
+    if (debugmode == 0 && !getenv("CYRUS_ISDAEMON")) {
 
-    oldumask = umask((mode_t) 0); /* for Linux */
+	pid = fork();
 
-    if (bind(s, (struct sockaddr *)&local, len) == -1) {
-	perror("bind");
-	cyrus_done();
-	exit(1);
+	if (pid == -1) {
+	    perror("fork");
+	    exit(1);
+	}
+
+	if (pid != 0) { /* parent */
+	    exit(0);
+	}
     }
-    umask(oldumask); /* for Linux */
-    chmod(local.sun_path, 0777); /* for DUX */
+    /* child */
+
 
     /* get ready for select() */
     FD_ZERO(&read_set);
@@ -362,13 +348,14 @@ int main(int argc, char **argv)
     for (;;) {
 	int n;
 
+	signals_poll();
+
 	/* check for shutdown file */
-	if ((fd = open(shutdownfilename, O_RDONLY, 0)) != -1) {
+	if (shutdown_file(NULL, 0)) {
 	    /* signal all processes to shutdown */
 	    if (verbose || debugmode)
-		syslog(LOG_DEBUG, "IDLE_ALERT\n");
-
-	    hash_enumerate(&itable, idle_alert, NULL);
+		syslog(LOG_DEBUG, "Detected shutdown file\n");
+	    shut_down(1);
 	}
 
 	/* timeout for select is 1 second */
@@ -377,7 +364,7 @@ int main(int argc, char **argv)
 
 	/* check for the next input */
 	rset = read_set;
-	n = select(nfds, &rset, NULL, NULL, &timeout);
+	n = signals_select(nfds, &rset, NULL, NULL, &timeout);
 	if (n < 0 && errno == EAGAIN) continue;
 	if (n < 0 && errno == EINTR) continue;
 	if (n == -1) {
@@ -387,29 +374,19 @@ int main(int argc, char **argv)
 	    fatal("select error",-1);
 	}
 
-	/* read on unix socket */
+	/* read and process a message */
 	if (FD_ISSET(s, &rset)) {
-	    fromlen = sizeof(from);
-	    n = recvfrom(s, (void*) &idledata, sizeof(idle_data_t), 0,
-			 (struct sockaddr *) &from, &fromlen);
+	    struct sockaddr_un from;
+	    idle_message_t msg;
 
-	    if (n > 0) {
-		if (n <= IDLEDATA_BASE_SIZE ||
-		    idledata.mboxname[n - 1 - IDLEDATA_BASE_SIZE] != '\0')
-		    syslog(LOG_ERR, "Invalid message received, size=%d\n", n);
-		else 
-		    process_msg(&idledata);
-	    }
-	} else {
-	    /* log some sort of error */	    
+	    if (idle_recv(&from, &msg))
+		process_message(&from, &msg);
 	}
 
     }
 
-    cyrus_done();
-
-    /* never gets here */      
-    exit(1);
+    /* NOTREACHED */
+    shut_down(1);
 }
 
 void printstring(const char *s __attribute__((unused)))
