@@ -450,7 +450,7 @@ int service_init(int argc __attribute__((unused)),
     }
 
     /* setup for sending IMAP IDLE notifications */
-    idle_enabled();
+    idle_init();
 
     /* Set namespace */
     if ((r = mboxname_init_namespace(&popd_namespace, 1)) != 0) {
@@ -667,6 +667,8 @@ void shut_down(int code)
     denydb_close();
     denydb_done();
 
+    idle_done();
+
     if (popd_in) {
 	prot_NONBLOCK(popd_in);
 	prot_fill(popd_in);
@@ -1503,6 +1505,11 @@ void cmd_pass(char *pass)
 	}
     }
 
+    /* popd_authstate may have been set as a side effect
+     * of sasl_checkpass() calling mysasl_proxy_policy */
+    if (popd_authstate)
+	auth_freestate(popd_authstate);
+
     popd_authstate = auth_newstate(popd_userid);
 
     openinbox();
@@ -1748,6 +1755,10 @@ int openinbox(void)
 	    IMAP_PERMISSION_DENIED : IMAP_MAILBOX_NONEXISTENT;
 	log_level = LOG_INFO;
     }
+    else if (!r && (mbentry.mbtype & MBTYPE_DELETED)) {
+	r = IMAP_MAILBOX_NONEXISTENT;
+	log_level = LOG_INFO;
+    }
     if (r) {
 	sleep(3);
 	syslog(log_level, "Unable to locate maildrop %s: %s",
@@ -1758,6 +1769,19 @@ int openinbox(void)
 	goto fail;
     }
 
+    if (mbentry.mbtype & MBTYPE_RESERVE) r = IMAP_MAILBOX_RESERVED;
+    if (mbentry.mbtype & MBTYPE_MOVING) r = IMAP_MAILBOX_MOVED;
+    if (r) {
+	sleep(3);
+	log_level = LOG_INFO;
+	syslog(log_level, "Unable to open maildrop %s: %s",
+	       inboxname, error_message(r));
+	prot_printf(popd_out,
+		    "-ERR [SYS/TEMP] Unable to open maildrop: %s\r\n",
+		    error_message(r));
+	goto fail;
+    }
+
     if (mbentry.mbtype & MBTYPE_REMOTE) {
 	/* remote mailbox */
 	char *server = mbentry.partition;
@@ -2055,7 +2079,7 @@ static int update_seen(void)
 
     /* we know this mailbox must be owned by the user, because 
      * all POP mailboxes are */
-    for (i = 0; i < popd_exists; i++) {
+    for (i = 1; i <= popd_exists; i++) {
 	if (!popd_msg[i].seen)
 	    continue; /* don't even need to check */
 	if (mailbox_read_index_record(popd_mailbox, popd_msg[i].recno, &record))
