@@ -38,8 +38,6 @@
  * WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER IN
  * AN ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING
  * OUT OF OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
- *
- * $Id: prot.c,v 1.100 2010/06/28 12:06:43 brong Exp $
  */
 
 #include <config.h>
@@ -228,9 +226,7 @@ int prot_sasldecode(struct protstream *s, int n)
  * Turn on SASL for this connection
  */
 
-int prot_setsasl(s, conn)
-struct protstream *s;
-sasl_conn_t *conn;
+int prot_setsasl(struct protstream *s, sasl_conn_t *conn)
 {
     const void *ssfp;
     int result;
@@ -628,7 +624,7 @@ int prot_fill(struct protstream *s)
 	    FD_SET(s->fd, &rfds);
 
 	    if (!haveinput &&
-		(select(s->fd + 1, &rfds, (fd_set *)0, (fd_set *)0,
+		(signals_select(s->fd + 1, &rfds, (fd_set *)0, (fd_set *)0,
 			&timeout) <= 0)) {
 		if (s->readcallback_proc) {
 		    (*s->readcallback_proc)(s, s->readcallback_rock);
@@ -673,7 +669,7 @@ int prot_fill(struct protstream *s)
 		timeout.tv_usec = 0;
 		FD_ZERO(&rfds);
 		FD_SET(s->fd, &rfds);
-		r = select(s->fd + 1, &rfds, (fd_set *)0, (fd_set *)0,
+		r = signals_select(s->fd + 1, &rfds, (fd_set *)0, (fd_set *)0,
 			   &timeout);
 		now = time(NULL);
 	    } while ((r == 0 || (r == -1 && errno == EINTR && !signals_poll())) &&
@@ -1547,7 +1543,7 @@ int prot_select(struct protgroup *readstreams, int extra_read_fd,
 	    timeout->tv_usec = 0;
 	}
 
-	if(select(max_fd + 1, &rfds, NULL, NULL, timeout) == -1)
+	if(signals_select(max_fd + 1, &rfds, NULL, NULL, timeout) == -1)
 	    return -1;
 
 	/* Reset now */
