@@ -340,10 +340,13 @@ int main(int argc, char **argv, char **envp)
     }
     id = atoi(p);
 
-    /* pick a random timeout between reuse_timeout -> 2*reuse_timeout
-     * to avoid massive IO overload if the network connection goes away */
     srand(time(NULL) * getpid());
-    reuse_timeout = reuse_timeout + (rand() % reuse_timeout);
+
+    /* if timeout is enabled, pick a random timeout between reuse_timeout
+     * and 2*reuse_timeout to avoid massive IO overload if the network
+     * connection goes away */
+    if (reuse_timeout)
+        reuse_timeout = reuse_timeout + (rand() % reuse_timeout);
 
     cyrus_init(alt_config, service, 0);
 
