@@ -39,7 +39,7 @@
  * AN ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING
  * OUT OF OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
  *
- * $Id: ptloader.c,v 1.49 2008/09/04 20:44:08 wescraig Exp $
+ * $Id: ptloader.c,v 1.50 2010/01/06 17:01:58 murch Exp $
  */
 
 #include <config.h>
@@ -62,7 +62,7 @@
 #include "hash.h"
 #include "global.h"
 #include "libconfig.h"
-#include "lock.h"
+#include "cyr_lock.h"
 #include "retry.h"
 #include "xmalloc.h"
 #include "ptloader.h"
@@ -125,11 +125,6 @@ const int config_need_data = 0;
 /* Globals */
 #define DB (config_ptscache_db)
 
-/* XXXXXXXXX */
-void des_init_random_number_generator() {
-        return;
-} 
-
 static char ptclient_debug = 0;
 struct db *ptsdb = NULL;
   
@@ -146,7 +141,7 @@ int service_init(int argc, char *argv[], char **envp __attribute__((unused)))
     /* set signal handlers */
     signal(SIGPIPE, SIG_IGN);
 
-    syslog(LOG_NOTICE, "starting: $Id: ptloader.c,v 1.49 2008/09/04 20:44:08 wescraig Exp $");
+    syslog(LOG_NOTICE, "starting: $Id: ptloader.c,v 1.50 2010/01/06 17:01:58 murch Exp $");
 
     while ((opt = getopt(argc, argv, "d:")) != EOF) {
 	switch (opt) {
